package stringAnalyzer;
/**
 * This class analyzes arbitrary Strings and determines various metrics
 *  such as vowel count, consonant count, and whitespace count.
 */

import java.util.regex.Pattern;
@SuppressWarnings("unused")

/**
 * @author hornick
 * original: 11/1/2012
 * last revision 10/7/2019
 */

public class Analyzer {
	private String text;  // string to be analyzed
	private int numVowels;	// number of vowels found
	private int numConsonants; // number of consonants found
	private int numBlanks; // number of blanks found
	private boolean state; // state of analysis (false initially)

	/**
	 * The public constructor for this class
	 * @param s the String to be analyzed
	 * @throws a NullPointerException if s is null 
	 * 		with message "null string passed to Analyzer constructor"
	 */
	public Analyzer(String s) {
			
		text = s;
		numVowels = 0;
		numConsonants = 0;
		numBlanks = 0;
		state = false; // indicates analyze() method below has not been called yet
		
	}
	
	/**
	 * This method analyzes the specified text string for the number of vowels, consonants, and blanks it contains.
     * Note: This method must be called FIRST.
	 * 
	 */
	public void analyze() {
		
		state = true; // indicates this method has been called
		
		// These are the vowel and consonant patterns used for matching characters in the String being analyzed.
		String vowels = new String("[aeiou]"); // [aeiou] is called a *regular expression*; means "any character within the brackets"
		String consonants = new String("[bcdfghjklmnpqrstvwxyz]");

		for( int i=0; i<text.length(); i++ ) { // iterate through the string, one character at a time

			if( Character.isSpaceChar(text.charAt(i)) ) // check for space character
				numBlanks++;

			CharSequence cs = text.subSequence(i, i+1); // extract a single character
			boolean isVowel = Pattern.matches(vowels, cs); // uses the Pattern class to match regular expressions
			if( isVowel )
				numVowels++;

			boolean isConsonant = Pattern.matches(consonants,  cs); // check for consonant
			if( isConsonant ) 
				numConsonants++;
		}
	}


	/**
	 * retrieves the number of vowels in the specified string
	 * @return number of vowels (>=0)
	 * @throws a IllegalStateException analyze() method above has not yet been called, 
	 * 		with message "analyze() method must be called first!"
	 */
	public int getVowelCount() {
		if(! state ) throw new IllegalStateException("analyze() method must be called first!");
		return numVowels;
	}

	/**
	 * retrieves the number of consonants in the specified string
	 * @return number of consonants (>=0)
	 */
	public int getConsonantCount() {
		return numConsonants;
	}

	/**
	 * retrieves the number of blanks in the specified string
	 * @return number of blanks (>=0)
	 */
	public int getSpaceCount() {
		return numBlanks;
	}

}
