package tests;

import static org.junit.Assert.*;

import java.io.IOException;

import org.junit.*;
import org.junit.rules.ExpectedException;

import stringAnalyzer.Analyzer;


/**
 * This  class is responsible for testing the Analyzer class.  Requires JUnit 5 or greater.
 * 
 * @version 1.0
 * @author hornick
 */

public class AnalyzerJUnitTests {

	/*
	 * The @Test annotation tells JUnit that the public void method to 
	 * which it is attached can be run as a test case. Before running the method, 
	 * JUnit constructs a fresh instance of this class, then invokes 
	 * each annotated @Test methods. 
	 * Any assert-method exceptions thrown by the test method(s) will be 
	 * reported by JUnit as a failure. If no exceptions are thrown, the test 
	 * is assumed to have succeeded.
	 */
	
	
	// This is an example JUnit test method.
	@Test
	public void testABCDE() {
		System.out.println("\t testing for input 'a b c d e' ");
		
		Analyzer a = new Analyzer("a b c d e");
		a.analyze();
		final int nExpectedConsonants = 3;
		
		// Call the method we want to validate for correct operation:
		int nActualConsonants = a.getConsonantCount();
		
		// Now make sure that the method works correctly:		
		/*Assert.*/assertEquals(nExpectedConsonants, nActualConsonants);

		//NOTE: Various OVERLOADED versions of assertEquals exist.
		// This one tests equality of INTEGER values!!!
		// See http://junit.sourceforge.net/javadoc/org/junit/Assert.html
	}

	

/*
	@Test
	// This unit test verifies the operation of the analyze method for an empty string
	public void testEmptyString()  
	{
		Analyzer a = new Analyzer("");
		final int nExpectedVowels = 0;
		final int nExpectedConsonants = 0;
		final int nExpectedSpaces = 0;
		int nVowels = a.getVowelCount();
		int nConsonants = a.getConsonantCount();
		int nBlanks = a.getSpaceCount();
		assertEquals(nExpectedVowels, nVowels);
		assertEquals(nExpectedConsonants, nConsonants);
		assertEquals(nExpectedSpaces, nBlanks);
	}

	@Test
	// This unit test verifies the operation of the analyze method for an string containing all possible vowels and consonants
	public void testFullString() 
	{
		Analyzer a = new Analyzer("abc def ghi jkl mno pqr stu vwx yz ABC DEF GHI JKL MNO PQR STU VWX YZ");
		final int nExpectedVowels = 10;
		final int nExpectedConsonants = 42;
		final int nExpectedSpaces = 17;
		int nVowels = a.getVowelCount();
		int nConsonants = a.getConsonantCount();
		int nBlanks = a.getSpaceCount();
		assertEquals(nExpectedVowels, nVowels);
		assertEquals(nExpectedConsonants, nConsonants);
		assertEquals(nExpectedSpaces, nBlanks);
	}	


	//	TODO: Implement this unit test to verify that a null string reference is correctly handled
	@Test // This test uses the try-catch idiom commonly used in JUnit 3
	public void testNullStringConstruction1() {
		System.out.println("\t testing for null-arg Exception using try-catch ");
		try {
			Analyzer a = new Analyzer(null); // null arg should cause an Exception
			fail(); // fail if we get here, because Analyzer constructor should have thrown an Exception
		} catch (NullPointerException e ) {
			assertTrue( e.getMessage().equals("null string passed to Analyzer constructor"));
		} catch (Exception e ) {
			fail(); // fail because this should never happen either
		}
	}

	@Test(expected= NullPointerException.class) // this test expects an Exception to be thrown somewhere
	public void testNullStringConstruction2() {
		System.out.println("\t testing for null-arg Exception using @Test(expected=Exception.class)  ");
		Analyzer a = new Analyzer(null);// null arg should cause an Exception
	}
*/
	@Rule
	public ExpectedException thrown = ExpectedException.none();
	@Test // this test expects an Exception with a specific message to be thrown
	public void testNullStringConstruction3()  {
		System.out.println("\t testing for null-arg Exception using ExpectedException ");
		thrown.expect(NullPointerException.class); // we expect an exception to be thrown in this test
		thrown.expectMessage("null string passed to Analyzer constructor"); // ...with a message string as indicated
		Analyzer a = new Analyzer(null);// null arg should cause an Exception
	}

}